/*
 *
 * happytimer - Alarm clock for the GNOME notification area
 *
 *  @Copyright (C) 2007-2011 JinCheol Kim (shriekout@gmail.com).
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <gtk/gtk.h>
#include <string.h>
#include <gst/gst.h>
#include <libnotify/notify.h>
#include <glib/gi18n.h>
#include <mxml.h>
#include <config.h>
#include "happytimer.h"
#include "callback.h"
#include "interface.h"
#include "timecheck.h"

static gboolean bus_call(GstBus *bus, GstMessage *msg, gpointer data)
{
	switch (GST_MESSAGE_TYPE(msg)) {
		case GST_MESSAGE_EOS:
			gst_element_set_state(GST_ELEMENT(varList.pipeline[(int)data]), GST_STATE_NULL);
			g_object_unref(GST_ELEMENT(varList.pipeline[(int)data]));
			varList.pipeline[(int)data] = NULL;
			if (rtplay_togglebutton) {
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rtplay_togglebutton), FALSE);
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tplay_togglebutton), FALSE);
			}
			break;
		case GST_MESSAGE_ERROR:
			break;
		default: break;
	}

	return TRUE;
}

void play_sound(int what, char *fn)
{
	GstBus *bus;
	gchar *uri;

	if (what == PRE_SND)
		uri = g_filename_to_uri(fn, NULL, NULL);
	else
		uri = g_filename_to_uri(prefer.sound_file[what], NULL, NULL);

	varList.pipeline[what] = gst_element_factory_make("playbin2", "playbin2");
	g_object_set(G_OBJECT(varList.pipeline[what]), "uri", uri, NULL);

	bus = gst_pipeline_get_bus(GST_PIPELINE(varList.pipeline[what]));
	gst_bus_add_watch(bus, bus_call, (gpointer)what);
	gst_object_unref(bus);

	gst_element_set_state(varList.pipeline[what], GST_STATE_PLAYING);

	g_free(uri);
}

gboolean timeCheck(gpointer data)
{
	time_t now;
	struct tm *ptr;
	char buf[5];
	gchar *temp;

	time(&now);
	ptr = localtime(&now);
	strftime(buf, 5, "%M%S", ptr);

	if ((prefer.rtsound) && (!strcmp(buf, "0000"))) {
		if (varList.pipeline[RT_SND]) {
			gst_element_set_state(GST_ELEMENT(varList.pipeline[RT_SND]), GST_STATE_NULL);
			g_object_unref(varList.pipeline[RT_SND]);
			varList.pipeline[RT_SND] = NULL;
		}

		if (prefer.rtsound)
			play_sound(RT_SND, NULL);
	}

	if (varList.counting && !varList.pause) {
		if (varList.count) {
			varList.count--;
		} else {
			if (!alarm_window)
				alarm_window = create_alarm_window();

			if (varList.pipeline[ARM_SND]) {
				gst_element_set_state(varList.pipeline[ARM_SND], GST_STATE_NULL);
				g_object_unref(varList.pipeline[ARM_SND]);
				varList.pipeline[ARM_SND] = NULL;
			}

			if (prefer.tsound)
				play_sound(ARM_SND, NULL);

			if (varList.repeat)
				varList.count = varList.timer_origin;
			else
				on_start_button_clicked(NULL, NULL);
		}

		temp = return_format_date(varList.count);
		gtk_label_set_text(GTK_LABEL(timerleft_label), temp);

		if ((prefer.notify_before) && (varList.count == prefer.notify_before)) {
			NotifyNotification *n;
			GdkPixbuf *pixbuf;
			char buf[80];

			sprintf(buf, _("%d minutes to go."), (prefer.notify_before/60));
			pixbuf = gdk_pixbuf_new_from_file(LOGO_FILE, NULL);

			notify_init("Basics");
			n = notify_notification_new(_("Attention in the \"Happytimer\"."), buf, NULL);
			notify_notification_set_image_from_pixbuf(n, pixbuf);
			notify_notification_set_timeout(n, NOTIFY_EXPIRES_DEFAULT);
			notify_notification_show(n, NULL);

			g_object_unref(G_OBJECT(n));
			g_object_unref(G_OBJECT(pixbuf));
		}

		if (varList.count) {
			gtk_status_icon_set_tooltip_text(happytimer_statusicon, temp);
		} else {
			gtk_status_icon_set_tooltip_text(happytimer_statusicon, _("Happytimer"));
		}

		g_free(temp);
	}

	return TRUE;
}

gchar *return_format_date(time_t t)
{
	char *buf;
	struct struct_t *date;

	buf = (char *)malloc(sizeof(char) * 9);

	date = return_struct_date(t);

	sprintf(buf, "%03d:%02d:%02d", date->hrs, date->mins, date->secs);
	free(date);	

	return buf;
}

struct struct_t *return_struct_date(time_t t)
{
	struct struct_t *date;
	int hrs, mins, secs, temp;

	date = malloc(sizeof(struct struct_t));

	hrs = (int)t / 3600;
	temp = (int)t % 3600;
	mins = temp / 60;
	secs = temp % 60;

	date->hrs = hrs;
	date->mins = mins;
	date->secs = secs;

	return date;
}

void save_preferences(char *name, char *attr_name, char *attr_value, char *value, char *old_attr_value, char *old_value, int what)
{
	mxml_node_t *tree;
	mxml_node_t *node = NULL, *child = NULL;;
	FILE *fp;

	fp = fopen(varList.config_file, "r");
	tree = mxmlLoadFile(NULL, fp, MXML_OPAQUE_CALLBACK);
	fclose(fp);

	switch (what) {
		case PRESET_ADD:
			node = mxmlFindElement(tree, tree, "preset_group", NULL, NULL, MXML_DESCEND);

			if (node) {
				child = mxmlNewElement(node, name);
				mxmlNewOpaque(child, value);
				mxmlElementSetAttr(child, attr_name, attr_value);
			}

			break;

		case EDIT:
			node = mxmlFindElement(tree, tree, name, attr_name, old_attr_value, MXML_DESCEND);

			if (node) {
				if (value)
					mxmlSetOpaque(node->child, (const char *)value);

				if (attr_name)
					mxmlElementSetAttr(node, attr_name, (const char *)attr_value);
			}

			break;

		case PRESET_DELETE:
			node = mxmlFindElement(tree, tree, name, attr_name, old_attr_value, MXML_DESCEND);

			while (node) {
				if (strcmp(node->child->value.opaque, old_value)) {
					node = mxmlFindElement(node, tree, name, attr_name, old_attr_value, MXML_DESCEND);
					continue;
				} else {
					mxmlDelete(node);
					break;
				}
			}

			break;

		case PRESET_EDIT:
			node = mxmlFindElement(tree, tree, name, attr_name, old_attr_value, MXML_DESCEND);

			while (node) {
				if (strcmp(node->child->value.opaque, old_value)) {
					node = mxmlFindElement(node, tree, name, attr_name, old_attr_value, MXML_DESCEND);
					continue;
				} else {
					mxmlElementSetAttr(node, attr_name, attr_value);
					mxmlSetOpaque(node->child, value);

					break;
				}
			}

			break;
		
		default:
			break;
	}

	fp = fopen(varList.config_file, "w");
	mxmlSaveFile(tree, fp, MXML_NO_CALLBACK);
	mxmlDelete(tree);
	fclose(fp);
}
